package com.bib.essensbestellungsverwaltung;

import java.util.ArrayList;
import java.util.List;

/**
 * A collection of functions loosely related to food management
 * Acts as an abstraction layer to the database
 * @author Malte Schulze Hobeling
 */
public class FoodMgr {
    /**
     * inserts a food int to the database and creates the food_restriction entries
     * @param food the food to be created
     * @return id of food or -1
     * @author Malte Schulze Hobeling
     */
    public static long createFood(Food food){
        String[] foodH = {"name","description","isDessert","food_typeid"};
        String[] food_restrictionH = {"foodid","allergyid"};
        String[] foodD = {food.getName(),food.getDescription(),(food.isDessert() ? "1" : "0"), String.valueOf(food.getFoodType().getId())};
        long id = Database.insert("food",foodH,foodD);
        if(food.getAllergies().size() > 0){
            String sId = String.valueOf(id);
            for (Allergy allergy : food.getAllergies()) {
                String[] food_restrictionD = {sId, String.valueOf(allergy.getId())};
                Database.insert("food_restriction",food_restrictionH, food_restrictionD);
            }
        }
        return id;
    }

    /**
     * inserts a food_plan into the database
     * @param foodPlan the foodPlan to be created
     * @return id of food_plan or -1
     * @author Malte Schulze Hobeling
     */
    public static long createFood_plan(FoodPlan foodPlan){
        String[] food_planH = {"date","food1","food2","dessert1","dessert2"};
        String[] food_planD = {foodPlan.getDate(),
                String.valueOf(foodPlan.getFoodVegan().getId()),
                String.valueOf(foodPlan.getFoodSecond().getId()),
                String.valueOf(foodPlan.getDessertVegan().getId()),
                String.valueOf(foodPlan.getDessertSecond().getId())};
        return Database.insert("food_plan",food_planH,food_planD);
    }

    /**
     * updates a food_plan into the database
     * @return number of rows affected or -1 on error
     * @author Johannes Kantz
     */
    public static long updateFood_plan(FoodPlan foodPlan){
        String[] food_planH = {"id","food1","food2","dessert1","dessert2"};
        String[] food_planD = {String.valueOf(foodPlan.getId()),
                String.valueOf(foodPlan.getFoodVegan().getId()),
                String.valueOf(foodPlan.getFoodSecond().getId()),
                String.valueOf(foodPlan.getDessertVegan().getId()),
                String.valueOf(foodPlan.getDessertSecond().getId())};
        return Database.update("food_plan",food_planH,food_planD);
    }

    /**
     * returns all non desserts or all desserts
     * @param isDessert true for only desserts false for non desserts
     * @return a list of all non desserts or all desserts
     * @author Malte Schulze Hobeling
     */
    public static List<Food> getFood(boolean isDessert){
        String[] foodH = {"isDessert"};
        String[] foodD = {(isDessert ? "1" : "0")};
        List<String> entries = Database.select("food",foodH,foodD);
        List<Food> foods = new ArrayList<>();
        for (String entry : entries) {
            String[] parts = entry.split(":");
            foods.add(getFoodById(Long.parseLong(parts[0])));
        }
        return foods;
    }

    /**
     * getFood but returns only vegan food
     * @param isDessert true for only desserts false for non desserts
     * @return a list of all vegan non desserts or all vegan desserts
     * @author Malte Schulze Hobeling
     */
    public static List<Food> getVeganFood(boolean isDessert){
        String[] foodH = {"isDessert","food_typeid"};
        String[] foodD = {(isDessert ? "1" : "0"),"1"};
        List<String> entries = Database.select("food",foodH,foodD);
        List<Food> foods = new ArrayList<>();
        for (String entry : entries) {
            String[] parts = entry.split(":");
            foods.add(getFoodById(Long.parseLong(parts[0])));
        }
        return foods;
    }

    /**
     * returns a food_plan for a day
     * @param date YYYY-MM-DD one day
     * @return food_plan for date
     * @author Malte Schulze Hobeling
     */
    public static FoodPlan getFoodPlan(String date){
        String[] food_planH = {"date"};
        String[] food_planD = {date};
        List<String> entry = Database.select("food_plan",food_planH,food_planD);
        if(entry.size() < 1){
            return null;
        }
        String[] parts = entry.get(0).split(":");
        Food foodVegan = getFoodById(Long.parseLong(parts[2]));
        Food foodSecond = getFoodById(Long.parseLong(parts[3]));
        Food dessertVegan = getFoodById(Long.parseLong(parts[4]));
        Food dessertSecond = getFoodById(Long.parseLong(parts[5]));
        boolean isSent = !parts[6].equals("0");
        return new FoodPlan(Long.parseLong(parts[0]),date,foodVegan,foodSecond,dessertVegan,dessertSecond,isSent);
    }

    /**
     * returns a FoodPlan for a given id or null if no unique id was found
     * @param id id of the FoodPlan
     * @return FoodPlan or null
     * @author Malte Schulze Hobeling
     */
    public static FoodPlan getFoodPlanById(long id){
        List<String> entry = Database.getEntryById("food_plan",id);
        if(entry.size() != 1){
            return null;
        }
        String[] parts = entry.get(0).split(":");
        Food foodVegan = getFoodById(Long.parseLong(parts[2]));
        Food foodSecond = getFoodById(Long.parseLong(parts[3]));
        Food dessertVegan = getFoodById(Long.parseLong(parts[4]));
        Food dessertSecond = getFoodById(Long.parseLong(parts[5]));
        boolean isSent = !parts[6].equals("0");
        return new FoodPlan(id,parts[1], foodVegan,foodSecond,dessertVegan,dessertSecond,isSent);
    }

    /**
     * returns a Food for a given id or null if no unique id was found
     * @param id id of the Food
     * @return Food or null
     * @author Malte Schulze Hobeling
     */
    public static Food getFoodById(long id){
        List<String> entry = Database.getEntryById("food",id);
        if(entry.size() != 1){
            return null;
        }
        String[] parts = entry.get(0).split(":");
        String name = parts[1];
        String description = parts[2];
        boolean isDessert;
        isDessert = parts[3].equals("0");
        FoodType foodType = getFoodTypeById(Long.parseLong(parts[4]));
        List<Allergy> allergies = getAllergies(id);
        return new Food(id,name,description,isDessert,foodType,allergies);
    }

    /**
     * returns a FoodType for a given id or null if no unique id was found
     * @param id id of the FoodType
     * @return FoodType or null
     * @author Malte Schulze Hobeling
     */
    public static FoodType getFoodTypeById(long id){
        List<String> entry = Database.getEntryById("food_type",id);
        if(entry.size() != 1){
            return null;
        }
        String[] typeParts = entry.get(0).split(":");
        return new FoodType(Long.parseLong(typeParts[0]),typeParts[1]);
    }

    /**
     * returns an Allergy for a given id or null if no unique id was found
     * @param id id of the Allergy
     * @return Allergy or null
     * @author Malte Schulze Hobeling
     */
    public static Allergy getAllergyById(long id){
        String[] allergyH = {"id"};
        String[] allergyD = {String.valueOf(id)};
        List<String> entry = Database.select("allergy",allergyH,allergyD);
        if(entry.size() != 1){
            return null;
        }
        String[] allergyParts = entry.get(0).split(":");
        return new Allergy(id,allergyParts[1],allergyParts[2]);
    }

    /**
     * returns all Allergies for a given foodId
     * @param foodId the foodId you want the allergies from
     * @return a List of Allergy, can be empty
     * @author Malte Schulze Hobeling
     */
    public static List<Allergy> getAllergies(long foodId){
        List<Allergy> allergies = new ArrayList<>();
        String[] restrictionsH = {"foodid"};
        String[] restrictionsD = {String.valueOf(foodId)};
        List<String> restrictions = Database.select("food_restriction",restrictionsH,restrictionsD);
        for (String restriction : restrictions) {
            String[] partsRestrictions = restriction.split(":");
            allergies.add(getAllergyById(Long.parseLong(partsRestrictions[2])));
        }
        return allergies;
    }

    /**
     * inserts the selected food into food_Selection if the food_plan has not been sent
     * @param food_selectionData childid, food_planid, foodid
     * @return id or -1
     * @author Malte Schulze Hobeling
     */
    public static long createFood_selection(String[] food_selectionData){
        String[] food_selectionH = {"childid","food_planid","foodid"};
        FoodPlan food_plan = getFoodPlanById(Long.parseLong(food_selectionData[1]));
        if(food_plan != null && !food_plan.isSent()){
            return Database.insert("food_selection",food_selectionH,food_selectionData);
        }else {
            return -1;
        }
    }

    /**
     * accumulates the selected food for a given day and locks the corresponding food_plan
     * @param date YYYY-MM-DD day
     * @return the accumulated orders
     * @author Malte Schulze Hobeling
     */
    public static List<String> getDayOrder(String date){
        List<String> orders = new ArrayList<>();
        FoodPlan food_plan = getFoodPlan(date);
        if(food_plan == null){
            return orders;
        }
        String sId = String.valueOf(food_plan.getId());
        String[] food_selectionH = {"food_planid","foodid"};
        Food[] foodArray = {
                food_plan.getFoodVegan(),
                food_plan.getFoodSecond(),
                food_plan.getDessertVegan(),
                food_plan.getDessertSecond()
        };
        for(int i = 0; i < 4; i++){
            String foodName = foodArray[i].getName();
            String[] food_selectionD = {sId, String.valueOf(foodArray[i].getId())};
            int count = Database.count("food_selection",food_selectionH,food_selectionD);
            orders.add(count+" X "+foodName);
        }
        String[] updateH = {"id","issent"};
        String[] updateD = {sId,"1"};
        if(Database.update("food_plan",updateH,updateD) < 0){
            System.out.println("Fehler");
        }
        return orders;
    }
}
